<?php

declare(strict_types=1);

namespace spec\Dedi\SyliusImageOptimizerPlugin\Media\Http\Controller;

use Dedi\SyliusImageOptimizerPlugin\Media\Http\Controller\ImageResolverInterface;
use Liip\ImagineBundle\Binary\BinaryInterface;
use Liip\ImagineBundle\Imagine\Cache\CacheManager;
use Liip\ImagineBundle\Imagine\Data\DataManager;
use Liip\ImagineBundle\Imagine\Filter\FilterManager;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\HttpFoundation\Request;

class ImageResolverOnTheFlySpec extends ObjectBehavior
{
    function let(
        CacheManager $cacheManager,
        DataManager $dataManager,
        FilterManager $filterManager
    ) {
        $this->beConstructedWith(
            $cacheManager,
            $dataManager,
            $filterManager
        );
    }

    function it_is_initializable()
    {
        $this->shouldImplement(ImageResolverInterface::class);
    }

    function it_is_not_store_if_image_found(
        CacheManager $cacheManager,
        DataManager $dataManager,
        FilterManager $filterManager,
        Request $request,
        ParameterBag $parameterBag
    ) {
        $request->query = $parameterBag;
        $parameterBag->get(Argument::any())->shouldBeCalled()->willReturn(12);
        $cacheManager->isStored(Argument::any(),Argument::any())->shouldBeCalled()->willReturn(true);
        $cacheManager->resolve(Argument::any(),Argument::any())->shouldBeCalled()->willReturn('http://localhost');

        $this->filter($request, 'foo', 'bar');

        $dataManager->find()->shouldNotBeCalled();
        $filterManager->applyFilter()->shouldNotBeCalled();
        $cacheManager->store()->shouldNotBeCalled();

        $cacheManager->resolve('bar_12_12', 'foo')->shouldBeCalled();
    }

    function it_is_store_if_image_not_found(
        CacheManager $cacheManager,
        DataManager $dataManager,
        FilterManager $filterManager,
        BinaryInterface $filteredBinary,
        BinaryInterface $binary,
        Request $request,
        ParameterBag $parameterBag
    ) {
        $request->query = $parameterBag;
        $parameterBag->get(Argument::any())->shouldBeCalled()->willReturn(12);
        $dataManager->find('foo','bar')->shouldBeCalled()->willReturn($binary);

        $filterManager->applyFilter($binary, 'foo', [
            'filters' => [
                'thumbnail' => [
                    'size' => [12, 12],
                    'mode' => 'outbound',
                    'allow_upscale' => false,
                ],
            ],
        ])->shouldBeCalled()->willReturn($filteredBinary);

        $cacheManager->resolve('bar_12_12', 'foo')->shouldBeCalled()->willReturn('http://localhost');

        $this->filter($request, 'foo', 'bar');

        $cacheManager->isStored('bar_12_12', 'foo')->shouldHaveBeenCalled()->willReturn(false);
        $cacheManager->store($filteredBinary, 'bar_12_12', 'foo')->shouldBeCalled();
    }
}
