<?php

declare(strict_types=1);

namespace Dedi\SyliusImageOptimizerPlugin\Twig\Extension;

use Symfony\Component\Templating\EngineInterface;

class RenderPictureExtension extends \Twig_Extension
{
    /**
     * @var EngineInterface
     */
    private $engine;

    private string $defaultCustomizeTemplate;

    private string $defaultTemplate;

    public function __construct(
        EngineInterface $engine,
        string $defaultTemplate,
        string $defaultCustomizeTemplate
    ) {
        $this->engine = $engine;
        $this->defaultTemplate = $defaultTemplate;
        $this->defaultCustomizeTemplate = $defaultCustomizeTemplate;
    }

    public function getFunctions()
    {
        return [
            new \Twig_Function('dedi_sylius_image_optimizer_for_image', [$this, 'renderImageOptimizerForImage'], [
                'is_safe' => ['html'],
            ]),
        ];
    }

    public function renderImageOptimizerForImage(
        string $image,
        string $alternativeText = '',
        array $runtimeConfigurationImage = null,
        ?string $template = null
    ): string {
        if ($runtimeConfigurationImage === null) {
            return $this->engine->render($template ?? $this->defaultTemplate, [
                'image' => $image,
                'alternativeText' => $alternativeText,
            ]);
        }

        return $this->engine->render($template ?? $this->defaultCustomizeTemplate, [
            'image' => $image,
            'alternativeText' => $alternativeText,
            'runtimeConfigurationImage' => $runtimeConfigurationImage,
        ]);
    }
}
